/* -*- c-file-style: "gnu"; indent-tabs-mode: nil; -*-
 * 
 * distcc -- A simple distributed compiler system
 *
 * Copyright (C) 2003 by Martin Pool <mbp@samba.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 * USA
 */

/**
 * @file
 * GTK+ 2.x monitor for distcc.
 **/

#include "config.h"

#include <stdint.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>

#include <gtk/gtk.h>
#include <libgnome/libgnome.h>
#include <libgnomeui/libgnomeui.h>

#include "types.h"
#include "rpc.h"
#include "io.h"
#include "distcc.h"
#include "trace.h"
#include "exitcode.h"

const char *rs_program_name = "distccmon-gnome";

enum {
  COLUMN_PID = 0,
  COLUMN_STATUS,
  COLUMN_FILE,
  COLUMN_HOST,
  NUM_COLUMNS
};


/**
 * Create a GtkTreeModel that will contain information about all
 * running processes.
 **/
static GtkTreeModel *dcc_gnome_make_proc_model (void)
{
  GtkListStore *proc_store;
  GtkTreeIter iter;
  
  /* Create a table for process status */
  proc_store = gtk_list_store_new (NUM_COLUMNS,
                                   G_TYPE_INT, G_TYPE_STRING,
                                   G_TYPE_STRING,
                                   G_TYPE_STRING);

  gtk_list_store_append (proc_store, &iter);
  gtk_list_store_set (proc_store, &iter,
                      COLUMN_PID, 42,
                      COLUMN_STATUS, "Connecting",
                      COLUMN_FILE, "hello.c",
                      COLUMN_HOST, "red",
                      -1);

  gtk_list_store_append (proc_store, &iter);
  gtk_list_store_set (proc_store, &iter,
                      0, 69,
                      1, "Sending", 
                      2, "bye.c",
                      COLUMN_HOST, "green",
                      -1);

  return GTK_TREE_MODEL (proc_store);
}


static GtkWidget *dcc_gnome_make_proc_view (GtkTreeModel *proc_model)
{
  GtkWidget *proc_tree;
  GtkCellRenderer   *renderer;
  GtkTreeViewColumn *column;

  /* Make cell renderer */
  renderer = gtk_cell_renderer_text_new ();

  proc_tree = gtk_tree_view_new_with_model (proc_model);
  gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (proc_tree), TRUE);

  /* Add PID */
  column = gtk_tree_view_column_new_with_attributes ("Pid",
                                                     renderer,
                                                     "text", COLUMN_PID,
                                                     NULL);
  gtk_tree_view_append_column (GTK_TREE_VIEW (proc_tree), column);

  /* Add Status */
  column = gtk_tree_view_column_new_with_attributes ("Status",
                                                     renderer,
                                                     "text", 1,
                                                     NULL);
  gtk_tree_view_append_column (GTK_TREE_VIEW (proc_tree), column);

  /* File */
  column = gtk_tree_view_column_new_with_attributes ("File",
                                                     renderer,
                                                     "text", 2,
                                                     NULL);
  gtk_tree_view_append_column (GTK_TREE_VIEW (proc_tree), column);
  
  /* Host */
  column = gtk_tree_view_column_new_with_attributes ("Host",
                                                     renderer,
                                                     "text", COLUMN_HOST,
                                                     NULL);
  gtk_tree_view_append_column (GTK_TREE_VIEW (proc_tree), column);

  return proc_tree;
}



static int dcc_gnome_make_window (void)
{
    GtkWidget *topbox, *proc_tree, *proc_scroll;
    GtkTreeModel *proc_model;
    GtkWidget *mainwin;

    /* Create the main window */
    mainwin = gnome_app_new (rs_program_name, "distcc Monitor");
    gtk_widget_set_usize (mainwin, 300, 400);
    gtk_window_set_policy (GTK_WINDOW (mainwin),
                           TRUE, /* allow shrink */
                           TRUE, /* allow grow */
                           FALSE); /* auto shrink */

    /* Quit when it's closed */
    g_signal_connect (GTK_OBJECT(mainwin), "delete-event", 
                      G_CALLBACK (gtk_main_quit), NULL);
    g_signal_connect (GTK_OBJECT(mainwin), "destroy", 
                      G_CALLBACK (gtk_main_quit), NULL);

    /* Create a vbox for the contents */
    topbox = gtk_vbox_new (FALSE, 0);
    gnome_app_set_contents (GNOME_APP (mainwin),
                            topbox);

    proc_model = dcc_gnome_make_proc_model ();
    proc_tree = dcc_gnome_make_proc_view (proc_model);
    proc_scroll = gtk_scrolled_window_new (NULL, NULL);
    
    /* no horizontal scrolling; let the table stretch */
    gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (proc_scroll),
                                    GTK_POLICY_NEVER,
                                    GTK_POLICY_AUTOMATIC);
    gtk_container_add (GTK_CONTAINER (proc_scroll), proc_tree);

    gtk_container_add (GTK_CONTAINER (topbox),
                       proc_scroll);
    
    /* Show the application window */
    gtk_widget_show_all (mainwin);
    
    return 0;
}



int main(int argc, char **argv)
{
    GnomeProgram *prog;

    prog = gnome_program_init (rs_program_name,
                               PACKAGE_VERSION,
                               LIBGNOMEUI_MODULE,
                               argc,
                               argv,
                               GNOME_PARAM_NONE);
    
    dcc_gnome_make_window ();

    /* Keep running until quit */
    gtk_main ();

    return 0;
}
